/* ILLexer.java is a generated file.  You probably want to
 * edit JavaLexer.lex to make changes.  Use JFlex to generate it.
 * To generate JavaLexer.java
 * Install <a href="http://jflex.de/">JFlex</a> v1.3.2 or later.
 * Once JFlex is in your classpath run<br>
 * <code>java JFlex.Main JavaLexer.lex</code><br>
 * You will then have a file called JavaLexer.java
 */

/*
 * This file is part of a <a href="http://ostermiller.org/syntax/">syntax
 * highlighting</a> package.
 * Copyright (C) 1999-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=Syntax+Highlighting
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package TextDigest;

import java.io.*;
import com.Ostermiller.Syntax.Lexer.*;

/** 
 * JavaLexer is a java lexer.  Created with JFlex.  An example of how it is used:
 *  <CODE>
 *  <PRE>
 *  JavaLexer shredder = new JavaLexer(System.in);
 *  JavaToken t;
 *  while ((t = shredder.getNextToken()) != null){
 *      System.out.println(t);
 *  }
 *  </PRE>
 *  </CODE>
 * The tokens returned should comply with the 
 * <A Href=http://java.sun.com/docs/books/jls/html/>Java 
 * Language Specification</A>.
 * @see JavaToken
 */ 

%%

%public
%class ILLexer
%implements Lexer
%function getNextToken
%type Token 

%{
    private int lastToken;
    private int nextState=YYINITIAL;
    
    /** 
     * next Token method that allows you to control if whitespace and comments are
     * returned as tokens.
     */
    public Token getNextToken(boolean returnComments, boolean returnWhiteSpace)throws IOException{
        Token t = getNextToken();
        while (t != null && ((!returnWhiteSpace && t.isWhiteSpace()) || (!returnComments && t.isComment()))){
            t = getNextToken();
        }
        return (t); 
    }
        
    /**
     * Prints out tokens from a file or System.in.
     * If no arguments are given, System.in will be used for input.
     * If more arguments are given, the first argument will be used as
     * the name of the file to use as input
     *
     * @param args program arguments, of which the first is a filename
     */
    public static void main(String[] args) {
        InputStream in;
        try {
            if (args.length > 0){
                File f = new File(args[0]);
                if (f.exists()){
                    if (f.canRead()){
                        in = new FileInputStream(f);
                    } else {
                        throw new IOException("Could not open " + args[0]);
                    }
                } else {
                    throw new IOException("Could not find " + args[0]);
                }
            } else {
                in = System.in;
            }
            JavaLexer shredder = new JavaLexer(in);
            Token t;
            while ((t = shredder.getNextToken()) != null) {
                if (t.getID() != ILToken.WHITE_SPACE){
                    System.out.println(t);
                }
            }
        } catch (IOException e){
            System.out.println(e.getMessage());
        }
    }  

    /**
     * Closes the current input stream, and resets the scanner to read from a new input stream.
	 * All internal variables are reset, the old input stream  cannot be reused
	 * (content of the internal buffer is discarded and lost).
	 * The lexical state is set to the initial state.
     * Subsequent tokens read from the lexer will start with the line, char, and column
     * values given here.
     *
     * @param reader The new input.
     * @param yyline The line number of the first token.
     * @param yychar The position (relative to the start of the stream) of the first token.
     * @param yycolumn The position (relative to the line) of the first token.
     * @throws IOException if an IOExecption occurs while switching readers.
     */
    public void reset(java.io.Reader reader, int yyline, int yychar, int yycolumn) throws IOException{
        yyreset(reader);
        this.yyline = yyline;
		this.yychar = yychar;
		this.yycolumn = yycolumn;
	}
%}

%line
%char
%full

BLANK=([ ])
TAB=([\t])
FF=([\f])
CR=([\r])
LF=([\n])
EOL=({CR}|{LF}|{CR}{LF})
WhiteSpace=({BLANK}|{TAB}|{FF}|{EOL})
AnyNonSeparator=([^\t\f\r\n\ \(\)\{\}\[\]\;\,\.\=\>\<\!\~\:\+\-\*\/\%])
Digit=([0-9])
Number=({Digit}+)
ILLetter=([a-zA-Z_])

Identifier=({ILLetter}({ILLetter}|{Digit})*)
ConstId=({Identifier})
ErrorIdentifier=({AnyNonSeparator}+)

ConstantDefinition=("def def")


TradCommentBegin=("/*")
DocCommentBegin=("/**")
NonTermStars=([^\*\/]*[\*]+[^\*\/])
TermStars=([\*]+[\/])
CommentText=((([^\*]*[\/])|{NonTermStars})*)
CommentEnd=([^\*]*{TermStars})
TradComment=({TradCommentBegin}{CommentText}{CommentEnd})
DocCommentEnd1=([^\/\*]{CommentText}{CommentEnd})
DocCommentEnd2=({NonTermStars}{CommentText}{CommentEnd})
DocComment=({DocCommentBegin}({DocCommentEnd1}|{DocCommentEnd2}|{TermStars}|[\/]))
OpenComment=({TradCommentBegin}{CommentText}([^\*]*)([\*]*))



%%

<YYINITIAL> "(" { 
    lastToken = ILToken.SEPARATOR_LPAREN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
    }
<YYINITIAL> ")" {
    lastToken = ILToken.SEPARATOR_RPAREN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "{" {
    lastToken = ILToken.SEPARATOR_LBRACE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "}" {
    lastToken = ILToken.SEPARATOR_RBRACE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "[" {
    lastToken = ILToken.SEPARATOR_LBRACKET;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "]" {
    lastToken = ILToken.SEPARATOR_RBRACKET;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> ";" {
    lastToken = ILToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "," {
    lastToken = ILToken.SEPARATOR_COMMA;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "." {
    lastToken = ILToken.SEPARATOR_PERIOD;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}

<YYINITIAL> "=" {
    lastToken = ILToken.OPERATOR_ASSIGN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> ">" {
    lastToken = ILToken.OPERATOR_GREATER_THAN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "<" {
    lastToken = ILToken.OPERATOR_LESS_THAN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "!" {
    lastToken = ILToken.OPERATOR_LOGICAL_NOT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> ":" {
    lastToken = ILToken.OPERATOR_COLON;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "+" {
    lastToken = ILToken.OPERATOR_ADD;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "-" {
    lastToken = ILToken.OPERATOR_SUBTRACT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "*" {
    lastToken = ILToken.OPERATOR_MULTIPLY;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "/" {
    lastToken = ILToken.OPERATOR_DIVIDE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "%" {
    lastToken = ILToken.OPERATOR_MOD;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}

<YYINITIAL> "==" {
    lastToken = ILToken.OPERATOR_EQUAL;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "<=" { 
    lastToken = ILToken.OPERATOR_LESS_THAN_OR_EQUAL;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> ">=" { 
    lastToken = ILToken.OPERATOR_GREATER_THAN_OR_EQUAL;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "!=" { 
    lastToken = ILToken.OPERATOR_NOT_EQUAL;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "||" { 
    lastToken = ILToken.OPERATOR_LOGICAL_OR;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "&&" { 
    lastToken = ILToken.OPERATOR_LOGICAL_AND;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}

<YYINITIAL> "bool" {
    lastToken = ILToken.RESERVED_WORD_BOOL;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "buffer" {
    lastToken = ILToken.RESERVED_WORD_BUFFER;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "delay" { 
    lastToken = ILToken.RESERVED_WORD_DELAY;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "define" { 
    lastToken = ILToken.RESERVED_WORD_DEFINE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "false" { 
    lastToken = ILToken.RESERVED_WORD_FALSE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "from" { 
    lastToken = ILToken.RESERVED_WORD_FROM;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "get" { 
    lastToken = ILToken.RESERVED_WORD_GET;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "if" { 
    lastToken = ILToken.RESERVED_WORD_IF;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "in" { 
    lastToken = ILToken.RESERVED_WORD_IN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "infty" { 
    lastToken = ILToken.RESERVED_WORD_INFTY;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "int" { 
    lastToken = ILToken.RESERVED_WORD_INT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "lenght" { 
    lastToken = ILToken.RESERVED_WORD_LENGHT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "main" { 
    lastToken = ILToken.RESERVED_WORD_MAIN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "process" { 
    lastToken = ILToken.RESERVED_WORD_PROCESS;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "prop" { 
    lastToken = ILToken.RESERVED_WORD_PROP;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "put" { 
    lastToken = ILToken.RESERVED_WORD_PUT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "size" { 
    lastToken = ILToken.RESERVED_WORD_SIZE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "stateset" { 
    lastToken = ILToken.RESERVED_WORD_STATESET;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "to" { 
    lastToken = ILToken.RESERVED_WORD_TO;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "true" { 
    lastToken = ILToken.RESERVED_WORD_TRUE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "urgent" { 
    lastToken = ILToken.RESERVED_WORD_URGENT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "var" { 
    lastToken = ILToken.RESERVED_WORD_VAR;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> "when" { 
    lastToken = ILToken.RESERVED_WORD_WHEN;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> {Identifier} { 
    lastToken = ILToken.IDENTIFIER;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}




<YYINITIAL> {Number} {
    /* At this point, the number we found could still be too large.
     * If it is too large, we need to return an error.
     * Java has methods built in that will decode from a string
     * and throw an exception the number is too large 
     */     
    String text = yytext();
    try {
        /* bigger negatives are allowed than positives.  Thus
         * we have to be careful to make sure a neg sign is preserved
         */
        if (lastToken == ILToken.OPERATOR_SUBTRACT){
            Integer.decode('-' + text);
        } else {
            Integer.decode(text);
        }
        lastToken = ILToken.LITERAL_INTEGER_DECIMAL;
    } catch (NumberFormatException e){
        lastToken = ILToken.ERROR_INTEGER_DECIMIAL_SIZE;
    }
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}


<YYINITIAL> ({WhiteSpace}+) { 
    lastToken = ILToken.WHITE_SPACE;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}

<YYINITIAL> {DocComment} {
    lastToken = ILToken.COMMENT_DOCUMENTATION;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> {TradComment} {
    lastToken = ILToken.COMMENT_TRADITIONAL;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}

<YYINITIAL> {ErrorIdentifier} { 
    lastToken = ILToken.ERROR_IDENTIFIER;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
<YYINITIAL> {OpenComment} { 
    lastToken = ILToken.ERROR_UNCLOSED_COMMENT;
    String text = yytext();
    ILToken t = (new ILToken(lastToken,text,yyline,yychar,yychar+text.length(),nextState));
    return (t);
}
